function [cfar,k_on_z,c14]=integr2_cfar_c14(zgrid,tlscale,sigw,fluxes,cref,...
    canopy_parms,ust,mol,selection_parms,c14_parms,p_fluxes,beta,km_on_z,dco2dt)
% integrates 'far field' concentration according to diffusion equation;
 % {k_on_z} - structure containing vertical profile of the diffusion
 % coefficient
global betta_;
 cfar(1)=cref;
hcanopy=canopy_parms.hcanopy;
Leaf_Vol=canopy_parms.Leaf_Vol;
Cd=canopy_parms.Cd;
Lc=1/Leaf_Vol/Cd;
Scc=canopy_parms.Scc;
St=canopy_parms.St;

if selection_parms.HF08_modif==true && selection_parms.HF08_inside==true
    betta_=beta;
    options = optimset('Display','off','Jacobian','on');
    zlow = fsolve(@HF08_lower_height,canopy_parms.hcanopy,options);
    if zlow<0 || zlow > canopy_parms.hcanopy
        'not valid zlow='
        zlow
        pause;
    end
end
if nargout==3
    ctracer(1)=0;
    ndim=size(zgrid,2);
    matr=sparse(zeros(ndim,ndim));
    matr(1,1)=1;
    rhs=zeros(ndim,1);
    rhs(1)=c14_parms.ubc;% zero if flux from ground;
    totflx=0;
end
ndim=size(zgrid,2);
sumsrte=0;
for i=2:ndim
    dz=zgrid(i-1)-zgrid(i);

    kdif=Kc_above_canopy_HF2008(0.5*(zgrid(i)+zgrid(i-1)),hcanopy,ust,beta,Scc,St,Leaf_Vol,Cd,mol);

    flux=0.5*(fluxes(i)+fluxes(i-1))-dco2dt(i);
    cfar(i)=cfar(i-1)+dz*flux/kdif;
    flux=0;%0.5*(p_fluxes(i)+p_fluxes(i-1));

    if nargout==3
        fii=0.5*(cfar(i)+cfar(i-1));
        K_dz=(kdif/dz);
        F_fi=0.5*flux/fii;
        ctracer(i)=((K_dz+F_fi)*ctracer(i-1)+c14_parms.Bq_flux)/(K_dz-F_fi);
        
        Sorate=0.5*(-p_fluxes(i+1)+p_fluxes(i-1))/dz;
        if Sorate~=0 && zgrid(i)-hcanopy > dz
            'Source rate ne 0 above canopy'
            'i='
            i
            'zgrid(i)='
            zgrid(i)
   %         pause;
        end
        totflx=totflx+Sorate*dz*ctracer(i)/cfar(i);
        matr(i,i)=K_dz/dz;
        matr(i,i-1)=-K_dz/dz;

        if i>2
             matr(i-1,i-1)=matr(i-1,i-1)+K_dz/dz;
             matr(i-1,i)=-K_dz/dz;
        end

    end
    

    if nargout > 1
        k_on_z.k(i-1)=kdif;
        k_on_z.z(i-1)=0.5*(zgrid(i)+zgrid(i-1));
    end
    
    if zgrid(i)<=hcanopy
        ilast=i;% finally this will be canopy height
        break;
    end
    continue;
end

%below canopy height
for i=ilast+1:ndim
    dz=zgrid(i-1)-zgrid(i);
    

    if selection_parms.HF08_inside==true
        %follow Harman & Finnigan inside canopy if necessary

        mult=2*beta^3*Lc*ust/Scc;
        ki1=mult*exp((zgrid(i)-hcanopy)/2/beta^2/Lc);
        ki2=mult*exp((zgrid(i-1)-hcanopy)/2/beta^2/Lc);
        kdif=2*ki1*ki2/(ki1+ki2)+0.0000015;%added molecular diffusivity
        if selection_parms.HF08_numer==true 
            kdif=km_on_z.k(i)/Scc;% precalculated during velocity integration
        end
        
        
        if selection_parms.HF08_modif==true
            if zgrid(i-1)<zlow
                kdif=0.4*ust*(zlow/hcanopy)*0.5*(zgrid(i)+zgrid(i-1));
            end
        end
    else
        ki1=tlscale(i)*sigw(i)^2;
        ki2=tlscale(i-1)*sigw(i-1)^2;
        kdif=2*ki1*ki2/(ki1+ki2);
    end
    
    flux=0.5*(fluxes(i)+fluxes(i-1))-dco2dt(i);
    cfar(i)=cfar(i-1)+dz*flux/kdif;
    
    flux=0;%0.5*(p_fluxes(i)+p_fluxes(i-1));
    if nargout==3
        fii=0.5*(cfar(i)+cfar(i-1));
        K_dz=(kdif/dz);
        F_fi=0.5*flux/fii;
        ctracer(i)=((K_dz+F_fi)*ctracer(i-1)+c14_parms.Bq_flux)/(K_dz-F_fi);
 
        if i==ndim
            matr(i,i)=K_dz;
            matr(i,i-1)=-K_dz;
            rhs(i,1)=c14_parms.Bq_flux;
        else
            Sorate=0.5*(-p_fluxes(i+1)+p_fluxes(i-1))/dz;
            totflx=totflx+Sorate*dz*ctracer(i)/cfar(i);
            matr(i,i)=K_dz/dz - 0.0224*Sorate/cfar(i);
            %matr(i,i)=K_dz/dz;
            matr(i,i-1)=-K_dz/dz;
            sumsrte=sumsrte+dz*0.0224*Sorate/cfar(i)
        end
           matr(i-1,i-1)=matr(i-1,i-1)+K_dz/dz;
           matr(i-1,i)=-K_dz/dz;

    end
end
%c14=ctracer;return;
if nargout==3
    c14=(matr\rhs)';% solve linear system of equations
end
return;
end