function [cnear]=integr_cnear(zgrid,tlscale,sigw,fluxes,htop)
% integrates 'near field' concentration according to Linearized Near Field
% (LNF) theory of Raupach, 1989 (see also Haverd, et.al., 2009, BLM)
if nargin<5
    upperz=100;
else
    upperz=htop;
end
for i=1:size(zgrid,2)
    if zgrid(i)>htop
        cnear(i)=0;
        continue;
    end
    fun = @(zs)Kernel(zs,zgrid(i),zgrid,sigw,fluxes,tlscale);
    cnear(i) = quadgk(fun,0.01,upperz);
end

return;
end

function [res]=Kernel(zs,z,zgrid,sigw,fluxes,tlscale)
% kernel function which according to LNF theory is to be integrated to
% obtain concentration

maxsz=size(zgrid,2);
for i=1:size(zs,1)
    for j=1:size(zs,2)
        % calculation S(zs)
        
        if zgrid(1)<zs(i,j) % we assume that grid fully covers canopy and sources are only inside canopy
            res(i,j)=0;
        else
            lfound=0;
            for l=1:maxsz
                if zgrid(l)<zs(i,j)
                    lfound=l;
                    break;
                end
            end
            if lfound==0
                Srczs=fluxes(maxsz)/zgrid(maxsz);
                sigwzs=sigw(maxsz);
                Tlzs=tlscale(maxsz)*zs(i,j)/zgrid(maxsz);% linearly tend to zero approaching ground
                %res(i,j)=Srczs*(knksi((z-zs(i,j))/sigwzs/Tlzs)+knksi((z+zs(i,j))/sigwzs/Tlzs))/sigwzs;
                res(i,j)=0;
            else
                Srczs=(fluxes(l-1)-fluxes(l))/(zgrid(l-1)-zgrid(l));
                sigwzs=sigw(l)*(zgrid(l-1)-zs(i,j))+sigw(l-1)*(zs(i,j)-zgrid(l));% linearly interp
                Tlzs=tlscale(l)*(zgrid(l-1)-zs(i,j))+tlscale(l-1)*(zs(i,j)-zgrid(l));
                res(i,j)=Srczs*(knksi((z-zs(i,j))/sigwzs/Tlzs)+knksi((z+zs(i,j))/sigwzs/Tlzs))/sigwzs;
            end
            
        end
        
        
            
    end
end
return;
end
function [res]=knksi(ksi,exact)

if nargin < 2 
    exact=false;
end
if exact==false
    % approximation according to (34) from Raupach,  1989. Q.J.R.Met.Soc.
    % (115) 609-632.
    res=-0.39894*log(1-exp(-abs(ksi)))-0.15623*exp(-abs(ksi));
else
    % integration of exact function according to Raupach,  1989. Q.J.R.Met.Soc.
    % (115) 609-632.
    fun = @(x)myfun(x,ksi);
    res = quad(fun,0.00001,100);
end
  
end

function [fres]=myfun(x,ksi)
  for i=1:size(x,1)
      for j=1:size(x,2)
        sigmax(i,j)=(2*(x(i,j)-1+exp(-x(i,j))))^0.5;
        fres(i,j)=exp(-x(i,j))*exp(-ksi^2/2/sigmax(i,j)^2)/sigmax(i,j)/(2*3.1416)^0.5;
      end
  end
    return;
end