function [veloc,k_on_z,err]=integr_velo(zgrid,canopy_parms,ust,mol,selection_parms,...
tlscale,sigw,Sfric,beta)
% integrates 'far field' concentration according to diffusion equation;
 % {k_on_z} - structure containing vertical profile of the diffusion
 % coefficient
%Ref. Harman & Finnigan, 2008
err=false;
hcanopy=canopy_parms.hcanopy;
Cd=canopy_parms.Cd; Leaf_Vol=canopy_parms.Leaf_Vol;
Lc=1/Leaf_Vol/Cd;
betan=canopy_parms.betan;

switch nargin==9
  case false
    dsp=3.8;
  if mol>0 
    beta=betan/fim_stab(dsp,mol);% correction for beta dependance on stability
  else
    beta=betan;
  end
 end
uref=ust/beta;
maxsz=size(zgrid,2);
veloc=zeros(1,maxsz);
for i=maxsz:-1:1
     if i~=maxsz
        dz=zgrid(i)-zgrid(i+1);
     else
         dz=zgrid(maxsz-1)-zgrid(maxsz);
     end
    
    
    if zgrid(i)<=hcanopy
        ilast=i;
        continue;
    end
    veloc(ilast)=uref;
    
    [kdif,err]=Km_above_canopy_HF2008(zgrid(i)-0.5*dz,hcanopy,ust,beta,Leaf_Vol,Cd,mol);
    if err
        veloc=[];k_on_z=[];
        return;
    end
    
    if kdif < 0
        'kdif < 0; i='
        i
        pause;
    end
    veloc(i)=veloc(i+1)+dz*(ust^2)/kdif;
    
    if veloc(i)<0
        'veloc(i)<0;i='
        i
        err=true;veloc=[];k_on_z=[];
        return;
        %pause;
        
    end

    if nargout > 1
        k_on_z.k(i)=kdif;
        k_on_z.z(i)=0.5*(zgrid(i)+zgrid(i+1));
    end
    continue;
 end
% integrating inside canopy
tau2=ust^2;

k_on_z.ilast=ilast;

if false 
    %selection_parms.HF08_inside~=true - for the user interface full velocity integration is skipped 
    %IK2018 in case of empirical parameterization; instead analytical HF08
    %profile is used
    
    for i=ilast:maxsz
        x0(i-ilast+1)=uref-uref*(hcanopy-zgrid(i))/hcanopy;
        kdif(i-ilast+1)=canopy_parms.Scc*tlscale(i)*sigw(i)^2;
        Sfr(i-ilast+1)=Sfric(i);
        if kdif(i-ilast+1)<0.005 %roughly correspond to diffusion coef. at height 0.1 m when ust=0.1
            kdif(i-ilast+1)=0.005;
        end

        if nargout > 1
          k_on_z.k(i)=kdif(i-ilast+1);
          k_on_z.z(i)=zgrid(i);
        end

    end
    
      [x,niter]=newtonsys(@uprofile_newton,x0',1e-6,10000,...
            ust,uref,zgrid(ilast:maxsz),kdif,Sfr,canopy_parms.z0);
        veloc(ilast:maxsz)=x';
    return;
end


if selection_parms.HF08_numer == false %&& selection_parms.HF08_inside==true  
    %IK2018 see comment in if block loop above    
  for i=ilast:maxsz-1;
     veloc(i)=uref*exp((zgrid(i)-hcanopy)/2/beta^2/Lc);
     if nargout > 1
        k_on_z.k(i)=beta*veloc(i)*2*beta^3*Lc;
        k_on_z.z(i)=zgrid(i);
     end
  end
  return;
end
if selection_parms.HF08_inside==true && selection_parms.HF08_numer == true
    for i=ilast:maxsz
        x0(i-ilast+1)=uref-uref*(hcanopy-zgrid(i))/hcanopy;
        Sfr(i-ilast+1)=Sfric(i);
    end
    
    lm=2*beta^3*Lc;
    [x,niter]=newtonsys(@uprofile_mlength_newton,x0',1e-6,10000,...
            ust,uref,zgrid(ilast:maxsz),lm,Sfr,canopy_parms.z0);
    veloc(ilast:maxsz)=x';
    
    for i=ilast+1:maxsz
        kdif=(lm^2)*(veloc(i-1)-veloc(i))/(zgrid(i-1)-zgrid(i));
        if nargout > 1
           k_on_z.k(i)=kdif;
           k_on_z.z(i)=0.5*(zgrid(i-1)+zgrid(i));
        end        
    end

end
return;
end