% designed to solve momentum equation (with 'mixing length' hypothesis)
% d(lm|dUdz|dU/dz)/dz=U^2/Lc by Newton method - @newtonsys
% subroutine; 
function [res]=uprofile_mlength_newton(x,flag,ust,uref,zz,lm,Sfr,z0)
carman=0.4;
Nz=size(x,1);
lm2=lm^2;
if isempty(flag)
    flag='';
end

switch lower(flag)
case ''
    res=zeros(Nz,1);
    res(1,1)=x(1,1)-uref;
    
    dz=zz(1)-zz(2);
    if11=(x(1,1)>x(2,1)); if12=(x(1,1)<=x(2,1));
    res(2,1)=(lm2/dz/dz)*if11*(x(1,1)^2-2*x(1,1)*x(2,1)+x(2,1)^2)+...
        (lm2/dz/dz)*if12*(-x(1,1)^2+2*x(1,1)*x(2,1)-x(2,1)^2) ...
        - ust^2;
    
    for i=3:Nz-1
        
        dz1=zz(i-1)-zz(i);
        dz2=zz(i)-zz(i+1);
        dz=0.5*(dz1+dz2);
        if11=(x(i-1,1)>x(i,1)); if12=(x(i-1,1)<=x(i,1));
        if21=(x(i,1)>x(i+1,1)); if22=(x(i,1)<=x(i+1,1));
        
        res(i,1)=(lm2/dz1/dz1/dz)*if11*(x(i-1,1)^2-2*x(i-1,1)*x(i,1)+x(i,1)^2)+...
            (lm2/dz1/dz1/dz)*if12*(-x(i-1,1)^2+2*x(i-1,1)*x(i,1)-x(i,1)^2)-...
             (lm2/dz2/dz2/dz)*if21*(x(i,1)^2-2*x(i,1)*x(i+1,1)+x(i+1,1)^2)-...
             (lm2/dz2/dz2/dz)*if22*(-x(i,1)^2+2*x(i,1)*x(i+1,1)-x(i+1,1)^2)-...
             Sfr(i)*x(i,1)^2;
    end
       mult=(carman/log(zz(Nz)/z0))^2;
       
       if11=(x(Nz-1,1)>x(Nz,1)); if12=(x(Nz-1,1)<=x(Nz,1));
       dz=zz(Nz-1)-zz(Nz);
       res(Nz,1)=(lm2/dz/dz)*if11*(x(Nz-1,1)^2-2*x(Nz-1,1)*x(Nz,1)+x(Nz,1)^2) + ...
           (lm2/dz/dz)*if12*(-x(Nz-1,1)^2+2*x(Nz-1,1)*x(Nz,1)-x(Nz,1)^2)-...
        (mult/dz)*x(Nz,1)^2;
    
    
case 'jacobian'
     res=sparse(zeros(Nz,Nz));
     res(1,1)=1;

    dz=zz(1)-zz(2);
    if11=(x(1,1)>x(2,1)); if12=(x(1,1)<=x(2,1));
    res(2,1)=(lm2/dz/dz)*if11*(2*x(1,1)-2*x(2,1))+...
        (lm2/dz/dz)*if12*(-2*x(1,1)+2*x(2,1));
    res(2,2)=(lm2/dz/dz)*if11*(0-2*x(1,1)+2*x(2,1))+...
        (lm2/dz/dz)*if12*(0+2*x(1,1)-2*x(2,1));
    
     for i=3:Nz-1
        dz1=zz(i-1)-zz(i);
        dz2=zz(i)-zz(i+1);
        dz=0.5*(dz1+dz2);
        if11=(x(i-1,1)>x(i,1)); if12=(x(i-1,1)<=x(i,1));
        if21=(x(i,1)>x(i+1,1)); if22=(x(i,1)<=x(i+1,1));
        
        res(i,i-1)=(lm2/dz1/dz1/dz)*if11*(2*x(i-1,1)-2*x(i,1)+0)+...
            (lm2/dz1/dz1/dz)*if12*(-2*x(i-1,1)+2*x(i,1)-0); 
        res(i,i+1)=0-...
             (lm2/dz2/dz2/dz)*if21*(0-2*x(i,1)+2*x(i+1,1))-...
             (lm2/dz2/dz2/dz)*if22*(0+2*x(i,1)-2*x(i+1,1));
        res(i,i)=(lm2/dz1/dz1/dz)*if11*(0-2*x(i-1,1)+2*x(i,1))+...
            (lm2/dz1/dz1/dz)*if12*(0+2*x(i-1,1)-2*x(i,1))-...
             (lm2/dz2/dz2/dz)*if21*(2*x(i,1)-2*x(i+1,1)+0)-...
             (lm2/dz2/dz2/dz)*if22*(-2*x(i,1)+2*x(i+1,1)-0)-...
             Sfr(i)*2*x(i,1);
     end

       mult=(carman/log((zz(Nz)-25*z0)/z0))^2;
       if11=(x(Nz-1,1)>x(Nz,1)); if12=(x(Nz-1,1)<=x(Nz,1));
       dz=zz(Nz-1)-zz(Nz);


       res(Nz,Nz-1)=(lm2/dz/dz)*if11*(2*x(Nz-1,1)-2*x(Nz,1)+0) + ...
           (lm2/dz/dz)*if12*(-2*x(Nz-1,1)+2*x(Nz,1)-0); 
       
       res(Nz,Nz)=(lm2/dz/dz)*if11*(0-2*x(Nz-1,1)+2*x(Nz,1)) + ...
           (lm2/dz/dz)*if12*(0+2*x(Nz-1,1)-2*x(Nz,1))-...
        (mult/dz)*2*x(Nz,1);

     
end

end
